
import "Turbine.Gameplay";
import "Turbine.UI.Extensions";
import "Turbine.Utils";

CombatQuickslotBar = class( Turbine.UI.Extensions.SimpleWindow );

function CombatQuickslotBar:Constructor()
	Turbine.UI.Extensions.SimpleWindow.Constructor( self );
	
	self.spacing = 5;
	self.quickslotCount = 0;
	self.quickslotSize = 36;
	self.quickslots = { };
	self.quickslotInfo = { };
	
	self.menuButton = Turbine.UI.Control()
	self.menuButton:SetParent( self );
	self.menuButton:SetBackground( "TurbinePlugins/CombatQuickslots/Resources/DownArrow.tga" );
	self.menuButton:SetSize( 14, 13 );
	
	self.menuButton.MouseDown = function( sender, args )
		local contextMenu = Turbine.UI.ContextMenu()
		
		local sizeMenu = Turbine.UI.MenuItem( "Size" );
		local sizeMenuItems = sizeMenu:GetItems();
		
		local i;
		
		for i = 1, 15, 1 do
			local sizeItem = Turbine.UI.MenuItem( tostring( i ) );
			
			if ( self.quickslotCount == i ) then
				sizeItem:SetChecked( true );
			end
			
			sizeItem.Click = function( sender, args )
				local newCount = tonumber( sender:GetText() );
				self:SetQuickslotCount( newCount );
				self.settings.QuickslotCount = newCount;
				self:SaveSettings();
			end
			
			sizeMenuItems:Add( sizeItem );
		end
		
		local contextMenuItems = contextMenu:GetItems();
		contextMenuItems:Add( sizeMenu );
		contextMenu:ShowMenu();
	end
	
	self:LoadSettings();
	
	self:SetQuickslotCount( self.settings.QuickslotCount );
	
	self:SetVisible( true );
	self:SetOpacity( 1 );

	-- Hook up the visibility of the quickslot bar to only show when
	-- the player is in combat.
	self.player = Turbine.Gameplay.LocalPlayer();
	
	self.player.InCombatChanged = function( sender, args )
		self:SetVisible( self.player:IsInCombat() );
	end
end

function CombatQuickslotBar:LoadSettings()
	self.settings = Turbine.PluginData.Load( Turbine.DataScope.Character, "CombatQuickslotBarSettings" );

	if ( type( self.settings ) ~= "table" ) then
		self.settings = { };
	end

	if ( not self.settings.QuickslotCount ) then
		self.settings.QuickslotCount = 5;
	end
	
	self.quickslotInfo = Turbine.PluginData.Load( Turbine.DataScope.Character, "CombatQuickslots" );

	if ( type( self.quickslotInfo ) ~= "table" ) then
		self.quickslotInfo = { };
	end
end

function CombatQuickslotBar:SaveSettings()
	Turbine.PluginData.Save( Turbine.DataScope.Character, "CombatQuickslotBarSettings", self.settings );
end

function CombatQuickslotBar:SetQuickslotCount( value )
	local oldCount = self.quickslotCount;
	local newCount = value;
	local i;
	
	self.quickslotCount = value;

	-- Remove any quickslots no longer required.
	for i = newCount + 1, oldCount, 1 do
		self.quickslots[i]:SetVisible( false );
		self.quickslots[i]:SetParent( nil );
		self.quickslots[i] = nil;
	end
	
	local function OnQuickslotChanged( sender, args )
		local shortcut = sender:GetShortcut();

		if ( shortcut ~= nil ) then
			self.quickslotInfo[sender.Index] = { };
			self.quickslotInfo[sender.Index].Type = shortcut:GetType();
			self.quickslotInfo[sender.Index].Data= shortcut:GetData();
		else
			self.quickslotInfo[sender.Index] = nil;
		end

		if ( self.loading ) then
			return;
		end

		-- Request for the quickslot info table to be saved.
		Turbine.PluginData.Save( Turbine.DataScope.Character, "CombatQuickslots", self.quickslotInfo );
	end
		
	-- Add any new quickslots that should be created.
	for i = oldCount + 1, newCount, 1 do
		self.quickslots[i] = Turbine.UI.Lotro.Quickslot();
		self.quickslots[i]:SetParent( self );
		self.quickslots[i]:SetSize( self.quickslotSize, self.quickslotSize );
		self.quickslots[i]:SetPosition( ( i - 1 ) * ( self.quickslotSize + self.spacing ), 0 );
		self.quickslots[i].Index = i;
		self.quickslots[i].ShortcutChanged = OnQuickslotChanged;

		local quickslotInfo = self.quickslotInfo[i];

		if ( quickslotInfo ~= nil ) then
			local shortcut = Turbine.UI.Lotro.Shortcut( quickslotInfo.Type, quickslotInfo.Data );
			self.quickslots[i]:SetShortcut( shortcut );
		end
	end

	local displayWidth = Turbine.UI.Display:GetWidth();
	local displayHeight = Turbine.UI.Display:GetHeight();

	local width = self.quickslotCount * self.quickslotSize + ( self.quickslotCount - 1 ) * self.spacing;

	self:SetSize( width, self.quickslotSize + 22 );
	self:SetPosition( ( displayWidth - width ) / 2, displayHeight * 2 / 3 );

	self.menuButton:SetPosition( ( width - self.menuButton:GetWidth() ) / 2, self.quickslotSize + 2 );
end
